﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using Microsoft.Phone.Controls;
using Petzold.Phone.Silverlight;

namespace ElementHandledDemo
{
    public partial class MainPage : PhoneApplicationPage
    {
        Dictionary<UIElement, Point> originDictionary = new Dictionary<UIElement,Point>();

        // Constructor
        public MainPage()
        {
            InitializeComponent();
        }

        void OnRectangleManipulationStarted(object sender,
                                    ManipulationStartedEventArgs args)
        {
            // Bring touched element to top
            Panel pnl = (args.ManipulationContainer as FrameworkElement).Parent as Panel;

            for (int i = 0; i < pnl.Children.Count; i++)
            {
                UIElement child = pnl.Children[i];
                Canvas.SetZIndex(child,
                    child == args.ManipulationContainer ? pnl.Children.Count : i);
            }

            // Save first ManipulationOrigin in dictionary
            originDictionary.Add(args.ManipulationContainer, args.ManipulationOrigin);            
            args.Handled = true;
        }

        void OnRectangleManipulationDelta(object sender,
                                          ManipulationDeltaEventArgs args)
        {
            // Get Transform and Matrix of touched element
            MatrixTransform xform =
                args.ManipulationContainer.RenderTransform as MatrixTransform;
            Matrix matx = xform.Matrix;

            // Get the delta scaling factors
            Point scale = args.DeltaManipulation.Scale;

            // Scaling factors are zero if only one finger is touching
            if (scale.X > 0 && scale.Y > 0)
            {
                Point origin = args.ManipulationOrigin;
                Point previousOrigin = originDictionary[args.ManipulationContainer];

                // Initialize the scaling center at the parent's origin
                double centerX = matx.OffsetX;
                double centerY = matx.OffsetY;

                // Apply the "magic formulas"
                if (scale.X != 1)
                    centerX += (scale.X * previousOrigin.X - origin.X) / (scale.X - 1);

                if (scale.Y != 1)
                    centerY += (scale.Y * previousOrigin.Y - origin.Y) / (scale.Y - 1);

                // Multiply scaling matrix. (ScaleAt is an extension method in MatrixExtensions.)
                matx = matx.ScaleAt(scale.X, scale.Y, centerX, centerY);
            }

            // Now apply translation.
            matx = matx.Translate(args.DeltaManipulation.Translation.X,
                                  args.DeltaManipulation.Translation.Y);

            // Set new Matrix on MatrixTransform
            xform.Matrix = matx;

            // Save the new ManipulationOrigin in dictionary
            originDictionary[args.ManipulationContainer] = args.ManipulationOrigin;
            args.Handled = true;
        }

        void OnRectangleManipulationCompleted(object sender, ManipulationCompletedEventArgs args)
        {
            // Remove element from dictionary
            originDictionary.Remove(args.ManipulationContainer);            
            args.Handled = true;
        }
    }
}
