﻿// MatrixExtensions.cs, Charles Petzold, July 2010
using System;
using System.Windows;
using System.Windows.Media;

namespace Petzold.Phone.Silverlight
{
    public static class MatrixExtensions
    {
        #region Non-Extension Methods

        public static Matrix Multiply(Matrix A, Matrix B)
        {
            return new Matrix(A.M11 * B.M11 + A.M12 * B.M21,
                              A.M11 * B.M12 + A.M12 * B.M22,
                              A.M21 * B.M11 + A.M22 * B.M21,
                              A.M21 * B.M12 + A.M22 * B.M22,
                              A.OffsetX * B.M11 + A.OffsetY * B.M21 + B.OffsetX,
                              A.OffsetX * B.M12 + A.OffsetY * B.M22 + B.OffsetY);
        }

        public static Matrix CreateTranslation(double x, double y)
        {
            return new Matrix(1, 0, 0, 1, x, y);
        }

        public static Matrix CreateScale(double scaleX, double scaleY)
        {
            return new Matrix(scaleX, 0, 0, scaleY, 0, 0);
        }

        public static Matrix CreateScale(double scaleX, double scaleY, double centerX, double centerY)
        {
            Matrix matx = CreateTranslation(-centerX, -centerY);
            matx = Multiply(matx, CreateScale(scaleX, scaleY));
            return Multiply(matx, CreateTranslation(centerX, centerY));
        }

        public static Matrix CreateRotation(double degrees)
        {
            double radians = Math.PI * degrees / 180;
            double cos = Math.Cos(radians);
            double sin = Math.Sin(radians);
            return new Matrix(cos, sin, -sin, cos, 0, 0);
        }

        public static Matrix CreateRotation(double degrees, double centerX, double centerY)
        {
            Matrix matx = CreateTranslation(-centerX, -centerY);
            matx = Multiply(matx, CreateRotation(degrees));
            return Multiply(matx, CreateTranslation(centerX, centerY));
        }

        #endregion

        #region Extension Methods

        public static Matrix Append(this Matrix A, Matrix B)
        {
            return Multiply(A, B);
        }

        public static Matrix Translate(this Matrix A, double x, double y)
        {
            return Multiply(A, CreateTranslation(x, y));
        }

        public static Matrix Scale(this Matrix A, double scaleX, double scaleY)
        {
            return Multiply(A, CreateScale(scaleX, scaleY));
        }

        public static Matrix ScaleAt(this Matrix A, double scaleX, double scaleY, double centerX, double centerY)
        {
            return Multiply(A, CreateScale(scaleX, scaleY, centerX, centerY));
        }

        public static Matrix Rotate(this Matrix A, double degrees)
        {
            return Multiply(A, CreateRotation(degrees));
        }

        public static Matrix RotateAt(this Matrix A, double degrees, double centerX, double centerY)
        {
            return Multiply(A, CreateRotation(degrees, centerX, centerY));
        }

        #endregion 
    }
}
